import { AppDataSource } from "../data-source";
import { Times } from "../entity/Times";
import { NextFunction, Request, Response } from "express"
import { Web } from "../entity/Web";

export class HotTopController {
    private TimesRepository = AppDataSource.getRepository(Times)
    async all(request: Request, response: Response, next: NextFunction) {
        const name = request.params.name
        const searchWeb = await AppDataSource.getRepository(Web).findOne({
            where:{
                name:name
            }
        })
        if(searchWeb == null){
            response.status(404)
            return "error"
        }
        const nowDate = new Date()
        const yestDay = new Date()
        let hours = 1
        let rawHours = parseInt(request.query.hours as unknown as string)
        if (request.query.hours !=undefined && !isNaN(rawHours) && rawHours <= 24 && rawHours > 0) {
            hours = parseInt(request.query.hours as unknown as string)
        }
        else {
            response.status(404)
            return "error"
        }

        yestDay.setHours(nowDate.getHours() - hours)
        // console.log(yestDay.toLocaleString())
        let limit: number
        if (request.query.limit == undefined) {
            response.status(404)
            return "error"
        }
        limit = Number.parseInt(request.query.limit as string)
        if (isNaN(limit) || limit > 50 || limit < 1) {
            response.status(404)
            return "error"
        }

        // console.log(limit,hours)
        // return "OK"
        return this.TimesRepository
            .createQueryBuilder("times")
            .leftJoinAndSelect("times.fromWeb", "fromWeb")
            .where("times.lastTime BETWEEN :beginTime AND :endTime",
                {
                    beginTime: yestDay,
                    endTime: nowDate
                }
            )
            .andWhere("times.fromWeb = :searchWeb",{
                searchWeb:searchWeb.id
            })
            .addOrderBy('times.times', "DESC")
            .addOrderBy('times.lastTime', "DESC")
            .limit(limit)
            // .getSql()
            .getMany()
            
    }
}